@@ -20,17 +20,28 @@
  *
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include "gmime-message.h"
-#include "gmime-utils.h"
-#include "internet-address.h"
 #include <string.h>
 #include <ctype.h>
 #include <locale.h>
 
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
+#include "gmime-message.h"
+#include "gmime-utils.h"
+#include "gmime-stream-mem.h"
+
+static void g_mime_message_destroy (GMimeObject *object);
+
+static GMimeObject object_template = {
+	0, 0, g_mime_message_destroy
+};
 
 static char *rfc822_headers[] = {
 	"Return-Path",
@@ -48,18 +59,26 @@ static char *rfc822_headers[] = {
 
 /**
  * g_mime_message_new: Create a new MIME Message object
+ * @pretty_headers: make pretty headers 
  *
- * Returns an empty MIME Message object with no headers nor content
- * set by default.
+ * If @pretty_headers is %TRUE, then the standard rfc822 headers are
+ * initialized so as to put headers in a nice friendly order. This is
+ * strictly a cosmetic thing, so if you are unsure, it is safe to say
+ * no (%FALSE).
+ *
+ * Returns an empty MIME Message object.
  **/
 GMimeMessage *
-g_mime_message_new ()
+g_mime_message_new (gboolean pretty_headers)
 {
 	GMimeMessage *message;
 	GMimeHeader *headers;
 	int i;
 	
 	message = g_new0 (GMimeMessage, 1);
+	g_mime_object_construct (GMIME_OBJECT (message),
+				 &object_template,
+				 GMIME_MESSAGE_TYPE);
 	
 	message->header = g_new0 (GMimeMessageHeader, 1);
 	
@@ -67,47 +86,31 @@ g_mime_message_new ()
 	
 	message->header->headers = headers = g_mime_header_new ();
 	
-	/* Populate with the "standard" rfc822 headers so we can have a standard order */
-	for (i = 0; rfc822_headers[i]; i++) 
-		g_mime_header_set (headers, rfc822_headers[i], NULL);
-			   
+	if (pretty_headers) {
+		/* Populate with the "standard" rfc822 headers so we can have a standard order */
+		for (i = 0; rfc822_headers[i]; i++) 
+			g_mime_header_set (headers, rfc822_headers[i], NULL);
+	}
+	
 	return message;
 }
 
 static gboolean
 recipients_destroy (gpointer key, gpointer value, gpointer user_data)
 {
-	GList *recipients = value;
+	InternetAddressList *recipients = value;
 	
-	if (recipients) {
-		GList *recipient;
-		
-		recipient = recipients;
-		while (recipient) {
-			internet_address_destroy (recipient->data);
-			recipient = recipient->next;
-		}
-		
-		g_list_free (recipients);
-	}
+	internet_address_list_destroy (recipients);
 	
 	return TRUE;
 }
 
-
-/**
- * g_mime_message_destroy: Destroy the MIME Message.
- * @message: MIME Message to destroy
- *
- * Releases all memory used by the MIME Message and it's child MIME
- * Parts back to the Operating System for reuse.
- **/
-void
-g_mime_message_destroy (GMimeMessage *message)
+static void
+g_mime_message_destroy (GMimeObject *object)
 {
-	int i;
+	GMimeMessage *message = (GMimeMessage *) object;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (object));
 	
 	g_free (message->header->from);
 	g_free (message->header->reply_to);
@@ -124,16 +127,16 @@ g_mime_message_destroy (GMimeMessage *message)
 	
 	g_free (message->header);
 	
-	/* destroy all child mime parts */
+	/* unref child mime part */
 	if (message->mime_part)
-		g_mime_part_destroy (message->mime_part);
+		g_mime_object_unref (GMIME_OBJECT (message->mime_part));
 	
 	g_free (message);
 }
 
 
 /**
- * g_mime_message_set_sender: Set the name/address of the sender
+ * g_mime_message_set_sender:
  * @message: MIME Message to change
  * @sender: The name and address of the sender
  *
@@ -141,11 +144,11 @@ g_mime_message_destroy (GMimeMessage *message)
  * (ex: "\"Joe Sixpack\" <joe@sixpack.org>")
  **/
 void
-g_mime_message_set_sender (GMimeMessage *message, const gchar *sender)
+g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 {
 	GMimeMessageHeader *header;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	header = message->header;
 	
@@ -158,33 +161,35 @@ g_mime_message_set_sender (GMimeMessage *message, const gchar *sender)
 
 
 /**
- * g_mime_message_get_sender: Get the name/address of the sender
+ * g_mime_message_get_sender:
  * @message: MIME Message
  *
+ * Gets the email address of the sender from @message.
+ *
  * Returns the sender's name and address of the MIME Message.
  **/
-const gchar *
+const char *
 g_mime_message_get_sender (GMimeMessage *message)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	return message->header->from;
 }
 
 
 /**
- * g_mime_message_set_reply_to: Set the Reply-To header of the MIME Message
+ * g_mime_message_set_reply_to:
  * @message: MIME Message to change
  * @reply_to: The Reply-To address
  *
  * Set the sender's Reply-To address on the MIME Message.
  **/
 void
-g_mime_message_set_reply_to (GMimeMessage *message, const gchar *reply_to)
+g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 {
 	GMimeMessageHeader *header;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	header = message->header;
 	
@@ -197,53 +202,42 @@ g_mime_message_set_reply_to (GMimeMessage *message, const gchar *reply_to)
 
 
 /**
- * g_mime_message_get_reply_to: Get the Reply-To address (if available)
+ * g_mime_message_get_reply_to:
  * @message: MIME Message
  *
+ * Gets the Reply-To address from @message.
+ *
  * Returns the sender's Reply-To address from the MIME Message.
  **/
-const gchar *
+const char *
 g_mime_message_get_reply_to (GMimeMessage *message)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	return message->header->reply_to;
 }
 
 
 static void
-sync_recipient_header (GMimeMessage *message, const gchar *type)
+sync_recipient_header (GMimeMessage *message, const char *type)
 {
-	GList *recipients;
+	InternetAddressList *recipients;
 	
 	/* sync the specified recipient header */
 	recipients = g_mime_message_get_recipients (message, type);
 	if (recipients) {
-		GString *recip;
+		char *string;
 		
-		recip = g_string_new ("");
-		while (recipients) {
-			InternetAddress *ia;
-			gchar *address;
-			
-			ia = recipients->data;
-			address = internet_address_to_string (ia, TRUE);
-			g_string_append (recip, address);
-			g_free (address);
-			
-			recipients = recipients->next;
-			if (recipients)
-				g_string_append (recip, ", ");
-		}
-		
-		g_mime_header_set (message->header->headers, type, recip->str);
-		g_string_free (recip, TRUE);
+		string = internet_address_list_to_string (recipients, TRUE);
+		g_mime_header_set (message->header->headers, type, string);
+		g_free (string);
 	} else
 		g_mime_header_set (message->header->headers, type, NULL);
 }
 
+
 /**
- * g_mime_message_add_recipient: Add a recipient
+ * g_mime_message_add_recipient:
  * @message: MIME Message to change
  * @type: Recipient type
  * @name: The recipient's name
@@ -254,24 +248,26 @@ sync_recipient_header (GMimeMessage *message, const gchar *type)
  * GMIME_RECIPIENT_TYPE_CC and GMIME_RECIPIENT_TYPE_BCC.
  **/
 void
-g_mime_message_add_recipient (GMimeMessage *message, gchar *type, const gchar *name, const gchar *address)
+g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *name, const char *address)
 {
+	InternetAddressList *recipients;
 	InternetAddress *ia;
-	GList *recipients;
 	
 	ia = internet_address_new_name (name, address);
 	
 	recipients = g_hash_table_lookup (message->header->recipients, type);
 	g_hash_table_remove (message->header->recipients, type);
 	
-	recipients = g_list_append (recipients, ia);
+	recipients = internet_address_list_append (recipients, ia);
+	internet_address_unref (ia);
+	
 	g_hash_table_insert (message->header->recipients, type, recipients);
 	sync_recipient_header (message, type);
 }
 
 
 /**
- * g_mime_message_add_recipients_from_string: Add a list of recipients
+ * g_mime_message_add_recipients_from_string:
  * @message: MIME Message
  * @type: Recipient type
  * @string: A string of recipient names and addresses.
@@ -283,11 +279,11 @@ g_mime_message_add_recipient (GMimeMessage *message, gchar *type, const gchar *n
  * specified in rfc822.
  **/
 void
-g_mime_message_add_recipients_from_string (GMimeMessage *message, gchar *type, const gchar *string)
+g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
 {
-	GList *recipients, *addrlist;
+	InternetAddressList *recipients, *addrlist;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (string != NULL);
 	
 	recipients = g_hash_table_lookup (message->header->recipients, type);
@@ -295,7 +291,7 @@ g_mime_message_add_recipients_from_string (GMimeMessage *message, gchar *type, c
 	
 	addrlist = internet_address_parse_string (string);
 	if (addrlist)
-		recipients = g_list_concat (recipients, addrlist);
+		recipients = internet_address_list_concat (recipients, addrlist);
 	
 	g_hash_table_insert (message->header->recipients, type, recipients);
 	
@@ -304,37 +300,39 @@ g_mime_message_add_recipients_from_string (GMimeMessage *message, gchar *type, c
 
 
 /**
- * g_mime_message_get_recipients: Get a list of recipients
+ * g_mime_message_get_recipients:
  * @message: MIME Message
  * @type: Recipient type
  *
+ * Gets a list of recipients of type @type from @message.
+ *
  * Returns a list of recipients of a chosen type from the MIME
  * Message. Available recipient types include:
  * GMIME_RECIPIENT_TYPE_TO, GMIME_RECIPIENT_TYPE_CC and
  * GMIME_RECIPIENT_TYPE_BCC.
  **/
-GList *
-g_mime_message_get_recipients (GMimeMessage *message, const gchar *type)
+InternetAddressList *
+g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	return g_hash_table_lookup (message->header->recipients, type);
 }
 
 
 /**
- * g_mime_message_set_subject: Set the message subject
+ * g_mime_message_set_subject:
  * @message: MIME Message
  * @subject: Subject string
  *
  * Set the Subject field on a MIME Message.
  **/
 void
-g_mime_message_set_subject (GMimeMessage *message, const gchar *subject)
+g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 {
 	GMimeMessageHeader *header;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	header = message->header;
 	
@@ -347,22 +345,24 @@ g_mime_message_set_subject (GMimeMessage *message, const gchar *subject)
 
 
 /**
- * g_mime_message_get_subject: Get the message subject
+ * g_mime_message_get_subject:
  * @message: MIME Message
  *
+ * Gets the message's subject.
+ *
  * Returns the Subject field on a MIME Message.
  **/
-const gchar *
+const char *
 g_mime_message_get_subject (GMimeMessage *message)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	return message->header->subject;
 }
 
 
 /**
- * g_mime_message_set_date: Set the message sent-date
+ * g_mime_message_set_date:
  * @message: MIME Message
  * @date: Sent-date (ex: gotten from time (NULL);)
  * @gmt_offset: GMT date offset (in +/- hours)
@@ -372,9 +372,9 @@ g_mime_message_get_subject (GMimeMessage *message)
 void
 g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset)
 {
-	gchar *date_str;
+	char *date_str;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	message->header->date = date;
 	message->header->gmt_offset = gmt_offset;
@@ -386,18 +386,18 @@ g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset)
 
 
 /**
- * g_mime_message_get_date: Get the message sent-date
+ * g_mime_message_get_date:
  * @message: MIME Message
  * @date: Sent-date
  * @gmt_offset: GMT date offset (in +/- hours)
  * 
  * Stores the date in time_t format in #date and the GMT offset in
- * #gmt_offset.
+ * @gmt_offset.
  **/
 void
 g_mime_message_get_date (GMimeMessage *message, time_t *date, int *gmt_offset)
 {
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	*date = message->header->date;
 	*gmt_offset = message->header->gmt_offset;
@@ -405,18 +405,20 @@ g_mime_message_get_date (GMimeMessage *message, time_t *date, int *gmt_offset)
 
 
 /**
- * g_mime_message_get_date_string: Get the message sent-date
+ * g_mime_message_get_date_string:
  * @message: MIME Message
+ *
+ * Gets the message's sent date in string format.
  * 
  * Returns the sent-date of the MIME Message in string format.
  **/
-gchar *
+char *
 g_mime_message_get_date_string (GMimeMessage *message)
 {
-	gchar *date_str;
-	gchar *locale;
+	char *date_str;
+	char *locale;
 	
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	locale = g_strdup (setlocale (LC_TIME, ""));
 	setlocale (LC_TIME, "POSIX");
@@ -440,11 +442,11 @@ g_mime_message_get_date_string (GMimeMessage *message)
  * Set the Message-Id on a message.
  **/
 void
-g_mime_message_set_message_id (GMimeMessage *message, const gchar *id)
+g_mime_message_set_message_id (GMimeMessage *message, const char *id)
 {
 	GMimeMessageHeader *header;
 	
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	
 	header = message->header;
 	
@@ -460,53 +462,79 @@ g_mime_message_set_message_id (GMimeMessage *message, const gchar *id)
  * g_mime_message_get_message_id: 
  * @message: MIME Message
  *
+ * Gets the Message-Id header of @message.
+ *
  * Returns the Message-Id of a message.
  **/
-const gchar *
+const char *
 g_mime_message_get_message_id (GMimeMessage *message)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	return message->header->message_id;
 }
 
 
 /**
- * g_mime_message_set_header: Add an arbitrary message header
+ * g_mime_message_add_header: Add an arbitrary message header
  * @message: MIME Message
- * @field: rfc822 header field
+ * @header: rfc822 header field
  * @value: the contents of the header field
  *
  * Add an arbitrary message header to the MIME Message such as X-Mailer,
  * X-Priority, or In-Reply-To.
  **/
 void
-g_mime_message_set_header (GMimeMessage *message, const gchar *field, const gchar *value)
+g_mime_message_add_header (GMimeMessage *message, const char *header, const char *value)
 {
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (header != NULL);
 	
-	g_mime_header_set (message->header->headers, field, value);
+	g_mime_header_add (message->header->headers, header, value);
+}
+
+
+/**
+ * g_mime_message_set_header: Add an arbitrary message header
+ * @message: MIME Message
+ * @header: rfc822 header field
+ * @value: the contents of the header field
+ *
+ * Set an arbitrary message header to the MIME Message such as X-Mailer,
+ * X-Priority, or In-Reply-To.
+ **/
+void
+g_mime_message_set_header (GMimeMessage *message, const char *header, const char *value)
+{
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (header != NULL);
+	
+	g_mime_header_set (message->header->headers, header, value);
 }
 
 
 /**
  * g_mime_message_get_header:
  * @message: MIME Message
- * @field: rfc822 header field
+ * @header: rfc822 header field
  *
- * Returns the value of the requested header (or NULL if it isn't set)
+ * Gets the value of the requested header @header if it exists, or
+ * %NULL otherwise.
+ *
+ * Returns the value of the requested header (or %NULL if it isn't set)
  **/
-const gchar *
-g_mime_message_get_header (GMimeMessage *message, const gchar *field)
+const char *
+g_mime_message_get_header (GMimeMessage *message, const char *header)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	g_return_val_if_fail (header != NULL, NULL);
 	
-	return g_mime_header_get (message->header->headers, field);
+	return g_mime_header_get (message->header->headers, header);
 }
 
 
 /**
- * g_mime_message_set_mime_part: Set the root-level MIME Part of the message
+ * g_mime_message_set_mime_part:
  * @message: MIME Message
  * @mime_part: The root-level MIME Part
  *
@@ -515,55 +543,66 @@ g_mime_message_get_header (GMimeMessage *message, const gchar *field)
 void
 g_mime_message_set_mime_part (GMimeMessage *message, GMimePart *mime_part)
 {
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (GMIME_IS_PART (mime_part));
+	
+	g_mime_object_ref (GMIME_OBJECT (mime_part));
 	
 	if (message->mime_part)
-		g_mime_part_destroy (message->mime_part);
+		g_mime_object_unref (GMIME_OBJECT (message->mime_part));
 	
 	message->mime_part = mime_part;
 }
 
 
 /**
- * g_mime_message_write_to_string: Write the MIME Message to a string
+ * g_mime_message_write_to_stream:
  * @message: MIME Message
- * @string: output string
+ * @stream: output stream
  *
  * Write the contents of the MIME Message to @string.
  **/
 void
-g_mime_message_write_to_string (GMimeMessage *message, GString *string)
+g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
 {
-	g_return_if_fail (message != NULL);
-	g_return_if_fail (string != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (stream != NULL);
 	
-	g_mime_header_write_to_string (message->header->headers, string);
+	g_mime_header_write_to_stream (message->header->headers, stream);
 	
-	if (message->mime_part)
-		g_mime_part_write_to_string (message->mime_part, TRUE, string);
-	else
-		g_string_append (string, "\n");
+	if (message->mime_part) {
+		g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
+		g_mime_part_write_to_stream (message->mime_part, stream);
+	} else
+		g_mime_stream_write (stream, "\n", 1);
 }
 
 
 /**
- * g_mime_message_to_string: Write the MIME Message to a string
+ * g_mime_message_to_string:
  * @message: MIME Message
  *
+ * Allocates a string buffer containing the mime message @message.
+ *
  * Returns an allocated string containing the MIME Message.
  **/
-gchar *
+char *
 g_mime_message_to_string (GMimeMessage *message)
 {
-	GString *string;
-	gchar *str;
+	GMimeStream *stream;
+	GByteArray *array;
+	char *str;
 	
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
-	string = g_string_new ("");
-	g_mime_message_write_to_string (message, string);
-	str = string->str;
-	g_string_free (string, FALSE);
+	array = g_byte_array_new ();
+	stream = g_mime_stream_mem_new ();
+	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), array);
+	g_mime_message_write_to_stream (message, stream);
+	g_mime_stream_unref (stream);
+	g_byte_array_append (array, "", 1);
+	str = array->data;
+	g_byte_array_free (array, FALSE);
 	
 	return str;
 }
@@ -598,14 +637,14 @@ g_mime_message_to_string (GMimeMessage *message)
  * message body. Note that this is an assumption and is thus not
  * guarenteed to always be correct.
  **/
-static gchar *
+static char *
 multipart_get_body (GMimePart *multipart, gboolean want_plain, gboolean *is_html)
 {
 	GMimePart *first = NULL;
-	const gchar *content;
-	gchar *body = NULL;
+	const char *content;
+	char *body = NULL;
 	GList *child;
-	guint len;
+	size_t len;
 	
 	child = multipart->children;
 	while (child) {
@@ -653,25 +692,31 @@ multipart_get_body (GMimePart *multipart, gboolean want_plain, gboolean *is_html
 
 
 /**
- * g_mime_message_get_body: Return the body of the message
+ * g_mime_message_get_body:
  * @message: MIME Message
  * @want_plain: request text/plain
  * @is_html: body returned is in html format
  *
+ * Attempts to get the body of the message in the preferred format
+ * specified by @want_plain.
+ *
  * Returns the prefered form of the message body. Sets the value of
- * #is_html to TRUE if the part returned is in HTML format, otherwise
+ * @is_html to TRUE if the part returned is in HTML format, otherwise
  * FALSE.
  * Note: This function is NOT guarenteed to always work as it
  * makes some assumptions that are not necessarily true. It is
  * recommended that you traverse the MIME structure yourself.
  **/
-gchar *
+char *
 g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gboolean *is_html)
 {
 	const GMimeContentType *type;
-	const gchar *content;
-	gchar *body = NULL;
-	guint len = 0;
+	const char *content;
+	char *body = NULL;
+	size_t len = 0;
+
+	g_return_val_if_fail (message!=NULL, NULL);
+	g_return_val_if_fail (is_html!=NULL, NULL);
 	
 	type = g_mime_part_get_content_type (message->mime_part);
 	if (g_mime_content_type_is_type (type, "text", "*")) {
@@ -693,15 +738,17 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 
 
 /**
- * g_mime_message_get_headers: Return the raw message headers
+ * g_mime_message_get_headers:
  * @message: MIME Message
  *
+ * Allocates a string buffer containing the raw message headers.
+ *
  * Returns an allocated string containing the raw message headers.
  **/
-gchar *
+char *
 g_mime_message_get_headers (GMimeMessage *message)
 {
-	g_return_val_if_fail (message != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
 	
 	return g_mime_header_to_string (message->header->headers);
 }
@@ -713,12 +760,12 @@ g_mime_message_get_headers (GMimeMessage *message)
  * @callback: function to call on each of the mime parts contained by the mime message
  * @data: extra data to pass to the callback
  *
- * Calls #callback on each of the mime parts in the mime message.
+ * Calls @callback on each of the mime parts in the mime message.
  **/
 void
 g_mime_message_foreach_part (GMimeMessage *message, GMimePartFunc callback, gpointer data)
 {
-	g_return_if_fail (message != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (callback != NULL);
 	
 	g_mime_part_foreach (message->mime_part, callback, data);
